@@ -10,7 +10,7 @@
 #        returns its error conditions, or upgrade to Perl 5.6.0 or
 #        greater, which has its own bugs.
 
-package HTML::Mason;
+package MyMason::MyApp;
 
 # Bring in main Mason package.
 use HTML::Mason;
@@ -19,7 +19,6 @@ use HTML::Mason;
 # Uncomment the second line (and comment the first) to use
 # Apache::Request instead of CGI.pm to parse arguments.
 use HTML::Mason::ApacheHandler;
-# use HTML::Mason::ApacheHandler qw(args_method=>'mod_perl');
 
 use strict;
 
@@ -36,20 +35,16 @@ use strict;
    use Apache::Session::File 1.50;
 }
 
-# Create Mason objects
+
+# Create ApacheHandler object
 #
-my $parser = new HTML::Mason::Parser;
-my $interp = new HTML::Mason::Interp (parser=>$parser,
-                                      comp_root=>'<component root>',
-                                      data_dir=>'<data directory>');
-my $ah = new HTML::Mason::ApacheHandler (interp=>$interp);
+my $ah = new HTML::Mason::ApacheHandler( comp_root => '<component root>',
+                                         data_dir => '<data directory>' );
 
 # Activate the following if running httpd as root (the normal case).
-# Resets ownership of all files created by Mason at startup. Change
-# these to match your server's 'User' and 'Group'.
+# Resets ownership of all files created by Mason at startup.
 #
-#chown ( (getpwnam('nobody'))[2], (getgrnam('nobody'))[2],
-#        $interp->files_written );
+#chown (Apache->server->uid, Apache->server->gid, $ah->interp->files_written);
 
 sub handler
 {
