@@ -18,17 +18,13 @@ C<perl -MCPAN -e 'install Bundle::HTML::Mason'>
 
 =head1 CONTENTS
 
-Data::Dumper
-
 File::Spec  0.8
 
-Params::Validate  0.13
+Params::Validate  0.16
 
-Apache::Request  0.31
+Exception::Class  1.01
 
-Exception::Class  1.00
-
-Cache::Cache  0.99
+Cache::Cache  1.00
 
 HTML::Mason
 
